@@ -1,25 +1,25 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 """Collection of helpers for using a PI device."""
+
 import sys
 import numbers
-from abc import abstractmethod
+from abc import abstractmethod, abstractproperty
 from collections import OrderedDict
 from io import open  # Redefining built-in 'open' pylint: disable=W0622
-from logging import debug
 from time import sleep, time
-from future.utils import raise_from
 
-from pipython.pidevice.common.gcscommands_helpers import isdeviceavailable
-from pipython.pidevice.gcs2.gcs2commands import GCS2Commands
-from pipython.pidevice.gcs21.gcs21commands import GCS21Commands
-from pipython.pidevice import gcserror
-from pipython.gcserror import GCSError
-from pipython.pidevice.gcs21.gcs21error import GCS21Error
-from pipython.pidevice.common.gcscommands_helpers import getgcsheader, getitemsvaluestuple
+from ...PILogger import PIDebug
+from ...pidevice.common.gcscommands_helpers import isdeviceavailable
+from ...pidevice.gcs2.gcs2commands import GCS2Commands
+from ...pidevice.gcs30.gcs30commands import GCS30Commands
+from ...pidevice import gcserror
+from ...gcserror import GCSError
+from ...pidevice.gcs30.gcs30error import GCS30Error
+from ...pidevice.common.gcscommands_helpers import getgcsheader, getitemsvaluestuple
 
 
-__signature__ = 0x140be928458e8e786db43234bbf5056
+__signature__ = 0x405aa2b789efb313fecb01205ca7007f
 
 # Class inherits from object, can be safely removed from bases in python3 pylint: disable=R0205
 class GCSBaseDeviceStartup(object):  # Too many instance attributes pylint: disable=R0902
@@ -30,7 +30,7 @@ class GCSBaseDeviceStartup(object):  # Too many instance attributes pylint: disa
         @type pitools : GCSBaseTools
         @param kwargs : Optional arguments with keywords that are passed to sub functions.
         """
-        debug('create an instance of GCSBaseDeviceStartup(kwargs=%s)', pitools.itemstostr(kwargs))
+        PIDebug('create an instance of GCSBaseDeviceStartup(kwargs=%s)', pitools.itemstostr(kwargs))
 
         if not isdeviceavailable([GCSBaseTools, ], pitools):
             raise TypeError('Type %s of pitools is not supported!' % type(pitools).__name__)
@@ -38,7 +38,6 @@ class GCSBaseDeviceStartup(object):  # Too many instance attributes pylint: disa
         self._pitools = pitools
         self._pidevice = pitools.pidevice
         self._refmodes = []
-        self._servo = None
         self._kwargs = kwargs
         self.prop = {'devname': self._pidevice.devname}
 
@@ -84,24 +83,21 @@ class GCSBaseDeviceStartup(object):  # Too many instance attributes pylint: disa
             self._refmodes = None
         else:
             self._refmodes = refmodes if isinstance(refmodes, list) else [refmodes] * len(self._pidevice.allaxes)
-        debug('GCSBaseDeviceStartup.refmodes = %s', self._pitools.itemstostr(self._refmodes))
+        PIDebug('GCSBaseDeviceStartup.refmodes = %s', self._pitools.itemstostr(self._refmodes))
 
+    @abstractproperty
     @property
     def servostates(self):
         """Servo states as dict {axis: state} or None."""
-        if isinstance(self._servo, bool):
-            return dict(list(zip(self._pidevice.axes, [self._servo] * self._pidevice.numaxes)))
-        return self._servo
 
+    @abstractproperty
     @servostates.setter
     def servostates(self, servo):
         """Desired servo states as boolean (for all stages) or dict {axis: state} or None to skip."""
-        self._servo = servo
-        debug('GCSBaseDeviceStartup.servostates = %s', self._pitools.itemstostr(self._servo))
 
     def stopall(self):
         """Stop all axes."""
-        debug('GCSBaseDeviceStartup.stopall()')
+        PIDebug('GCSBaseDeviceStartup.stopall()')
         self._pitools.stopall(**self._kwargs)
 
     @abstractmethod
@@ -116,7 +112,7 @@ class GCSRaise(object):  # Too few public methods pylint: disable=R0903
     """
 
     def __init__(self, gcserrorid, mustraise=True):
-        debug('create an instance of GCSRaise(gcserrorid=%s, mustraise=%s', gcserrorid, mustraise)
+        PIDebug('create an instance of GCSRaise(gcserrorid=%s, mustraise=%s', gcserrorid, mustraise)
         self.__expected = gcserrorid if isinstance(gcserrorid, (list, set, tuple)) else [gcserrorid]
         self.__mustraise = mustraise and gcserrorid
 
@@ -125,27 +121,26 @@ class GCSRaise(object):  # Too few public methods pylint: disable=R0903
 
     def __exit__(self, exctype, excvalue, _exctraceback):
         gcsmsg = '%r' % gcserror.translate_error(excvalue)
-        if exctype in (GCSError, GCS21Error):
+        if exctype in (GCSError, GCS30Error):
             if gcsmsg == str(excvalue.val):
-                gcsmsg = '%r' % GCS21Error.translate_error(excvalue)
+                gcsmsg = '%r' % GCS30Error.translate_error(excvalue)
             if excvalue in self.__expected:
-                debug('expected GCSError %s was raised', gcsmsg)
+                PIDebug('expected GCSError %s was raised', gcsmsg)
                 return True  # do not re-raise
         if not self.__mustraise and excvalue is None:
-            debug('no error was raised')
+            PIDebug('no error was raised')
             return True  # do not re-raise
         expected = ''
         for err in self.__expected:
             errval = gcserror.translate_error(err)
             if errval == str(err):
-                errval = GCS21Error.translate_error(err)
+                errval = GCS30Error.translate_error(err)
             expected = expected + errval + ', '
         msg = 'expected %s%r but raised was %s' % ('' if self.__mustraise else 'no error or ', expected, gcsmsg)
         if exctype is not None:
-            raise_from(ValueError(msg), exctype(excvalue))
-        else:
-            raise_from(ValueError(msg), Exception)
-        return True
+            raise ValueError(msg) from exctype(excvalue)
+
+        raise ValueError(msg) from Exception
 
 
 # Class inherits from object, can be safely removed from bases in python3 pylint: disable=R0205
@@ -173,7 +168,7 @@ class GCSBaseTools(object):
         """Provide a "ready to use" PI device.
         @type pidevice : pipython.gcscommands.GCSCommands
         """
-        if not isdeviceavailable([GCS2Commands, GCS21Commands, ], pidevice):
+        if not isdeviceavailable([GCS2Commands, GCS30Commands, ], pidevice.gcscommands):
             raise TypeError('Type %s of pidevice is not supported!' % type(pidevice).__name__)
 
         self._pidevice = pidevice
@@ -327,7 +322,7 @@ class GCSBaseTools(object):
                 self._pidevice.SVO(axis, states[i])
             except GCSError as exc:  # no GCSRaise() because we want to log a warning
                 if exc in toignore:
-                    debug('could not set servo for axis %r to %s: %s', axis, states[i], exc)
+                    PIDebug('could not set servo for axis %r to %s: %s', axis, states[i], exc)
                     success = False
                 else:
                     raise
@@ -389,7 +384,7 @@ class GCSBaseTools(object):
         @param header : Header information from qDRR() as dictionary or None.
         @param data : Datarecorder data as one or two dimensional list of floats or NumPy array.
         """
-        debug('save %r', filepath)
+        PIDebug('save %r', filepath)
         try:
             data = data.tolist()  # convert numpy array to list
         except AttributeError:
@@ -419,7 +414,7 @@ class GCSBaseTools(object):
         @return header : Header information from qDRR() as dictionary.
         @return data : Datarecorder data as two columns list of floats.
         """
-        debug('read %r', filepath)
+        PIDebug('read %r', filepath)
         headerstr, datastr = [], []
         gcsarray_found = False
 
@@ -506,11 +501,19 @@ class GCSBaseTools(object):
 
 
     @abstractmethod
-    def startup(self, stages=None, refmodes=None, servostates=True, **kwargs):
+    def startup(self, stages=None, refmodes=None, servostates=True, controlmodes=None, **kwargs):
         """Define 'stages', stop all, enable servo on all connected axes and reference them with 'refmodes'.
         Defining stages and homing them is done only if necessary.
         @param refmodes : Referencing command as string (for all stages) or list (not tuple!) or None to skip.
         @param servostates : Desired servo states as boolean (for all stages) or dict {axis: state} or None to skip.
+                         for controllers with GCS 3.0 syntax:
+                                if True the axis is switched into control mode 0x2.
+                                if False the axis is switched into contorl mode 0x1.
+        @param controlmodes : !!! Only valid for controllers with GCS 3.0 syntax !!!
+                          switches the axis into the given control mode
+                          int (for all stages) or dict {axis: controlmode} or None to ignore.
+                          To skip any control mode switch the servostate has to be None also!
+                          If controlmode is not None the parameter servostate is ignored
         @param kwargs : Optional arguments with keywords that are passed to sub functions.
         :return: Instance to a DeviceStarup object
         :rtype: DeviceStarup
@@ -530,7 +533,7 @@ class GCSBaseTools(object):
     def _get_closed_loop_on_target(self, axes, throwonaxiserror=False):
         """Return dictionary of on target states for closed loop 'axes'.
         @param axes : Axis or list/tuple of axes to get values for or None for all axes.
-        @param throwonaxiserror: only for GCS21! Throw an exeception on an axis error
+        @param throwonaxiserror: only for GCS30! Throw an exeception on an axis error
         @return : Dictionary of boolean ontarget states of 'axes'.
         """
 
